﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;
using System.IO;
using System.Drawing;
using WebApplication1.ImageryService;
using System.Net;
//using System.Drawing.Imaging;
namespace WebApplication1
{
    public partial class _Default : System.Web.UI.Page
    {
        private string _LoginName = "test";
        internal enum ProjectionType
        {
            ToMetric,
            ToDecimalDegree,
        }
        internal enum ImageryOverlayType : short
        {
            Satellite,
            Hybrid,
            Aerial,
            Road,
            RasterBitmap,   //Raster layer for bitmap
        }

        private string mstrLoginName = string.Empty;
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                InitializeMap();
                Map1.ZoomTo(GetCenterPoint(37.7072, -97.3175), 1126);
            }
        }

        private PointShape GetCenterPoint(double adblCenterLat, double adblCenterLng)
        {
            PointShape objCenterPt = (PointShape)ConvertShapeProjection(new PointShape(adblCenterLng, adblCenterLat)
                                            , ProjectionType.ToMetric);

            return objCenterPt;
        }


        private PointShape ConvertShapeProjection(BaseShape aobjShapeIn, ProjectionType aenumProjectionType)
        {
            PointShape objShapeOut = null;
            try
            {
                using (Proj4Projection objProj4 = new Proj4Projection())
                {
                    objProj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
                    objProj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
                    objProj4.Open();
                    objShapeOut = (PointShape)objProj4.ConvertToExternalProjection(aobjShapeIn);
                      
                    objProj4.Close();
                }
            }
            catch (Exception ex)
            {
                
            }
            return objShapeOut;
        }
        private void InitializeMap()
        {
            //Clear all existing drawn shape on the Edit Layer InMemory Layer
            Map1.EditOverlay.Features.Clear();
            Map1.CustomOverlays.Clear();
            Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#B3C6D4"));
            Map1.MapUnit = GeographyUnit.Meter;
            Map1.MapTools.MouseMapTool.Enabled = true;
            Map1.MapTools.MouseMapTool.IsMouseWheelDisabled = true;
            Map1.MapTools.Logo.Enabled = false;
            Map1.MapTools.PanZoomBar.Enabled = false;
            Map1.MapTools.PanZoomBar.IsGlobeButtonEnabled = false;
            Map1.MapTools.MouseCoordinate.Enabled = true;
            Map1.MapTools.ScaleLine.Enabled = true;
            Map1.MapTools.OverlaySwitcher.Enabled = true;
            Map1.ClientZoomLevelScales.Add(563);
            Map1.ClientZoomLevelScales.Add(281.5);
            Map1.CustomOverlays.Add(GetVEOverLayer("Satellite", _LoginName));
            Map1.CustomOverlays.Add(GetVEOverLayer("Hybrid", _LoginName));
            CreateAerialOverlay(null, false);
            CreateSketchOverlay();
            SyncClientZoomLevels();
            //Map1.OnClientDrawEnd = "OnClientDrawEnd";
            //Map1.OnClientEditEnd = "OnClientEditEnd";
            //Map1.OnClientBaseOverlayChanged = "OnClientBaseOverlayChanged";
            Map1.MapTools.OverlaySwitcher.BaseOverlayTitle = "Imagery View";

        }
        private void CreateAerialOverlay(RectangleShape CurrentExtent, bool IsActiveOverlay)
        {
            try
            {
                string OverlayName = "Aerial";
                LayerOverlay AerialOverLay = new LayerOverlay(OverlayName);
                if (Map1.CustomOverlays.Contains(OverlayName))
                {
                    Map1.CustomOverlays.Remove(OverlayName);
                }
                GdiPlusRasterLayer RasterImageLayer = new GdiPlusRasterLayer();
                RasterImageLayer.Name = OverlayName;
                string imageFilePath = (CurrentExtent == null) ? GetAerialDefaultImage : GetAerialExtentImage(CurrentExtent, Map1.Height.Value, Map1.Width.Value);
                GdiPlusRasterLayer gdiPlusImageLayer = new GdiPlusRasterLayer(imageFilePath);
                gdiPlusImageLayer.UpperThreshold = double.MaxValue;
                gdiPlusImageLayer.LowerThreshold = 0;
                AerialOverLay.Name = OverlayName;
                AerialOverLay.IsBaseOverlay = true;
                AerialOverLay.TileType = TileType.SingleTile;
                AerialOverLay.Layers.Add(gdiPlusImageLayer);
                AerialOverLay.IsVisibleInOverlaySwitcher = true;
                Map1.CustomOverlays.Add(AerialOverLay);
                if (IsActiveOverlay)
                {
                    Map1.ActiveBaseOverlay = AerialOverLay;
                }

            }
            catch (Exception ex)
            {
            }
        }
        internal static string GetAerialDefaultImage
        {
            get
            {
                string ImgPath = Path.Combine(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "images"), "AerialBlank");
                return Path.ChangeExtension(ImgPath, "jpg");
            }
        }
        internal string GetAerialExtentImage(RectangleShape currentExtent, double mapHeight, double mapWidth)
        {
            ExtentInfo extentInfo = new ExtentInfo();
            string strImageFilePath = string.Empty;
            try
            {
                extentInfo.UpperLeft = new Coord();
                extentInfo.LowerRight = new Coord();
                //Convert current map extent into ExtentInfo
                extentInfo.UpperLeft.X = currentExtent.UpperLeftPoint.X;
                extentInfo.UpperLeft.Y = currentExtent.UpperLeftPoint.Y;
                extentInfo.LowerRight.X = currentExtent.LowerRightPoint.X;
                extentInfo.LowerRight.Y = currentExtent.LowerRightPoint.Y;
                extentInfo.Height = (int)mapHeight;
                extentInfo.Width = (int)mapWidth;
                string strFilePath = Path.Combine(@"c:\ImageCacheFolder\", "Aerial");
                if (!Directory.Exists(strFilePath))
                {
                    Directory.CreateDirectory(strFilePath);
                }
                strImageFilePath = Path.ChangeExtension(Path.Combine(strFilePath, String.Format(@"{0}.{1}.{2}.{3}."
                                        , extentInfo.UpperLeft.X.ToString(), extentInfo.UpperLeft.Y.ToString()
                                        , extentInfo.LowerRight.X.ToString(), extentInfo.LowerRight.Y.ToString())), "jpg");
                if (!File.Exists(strImageFilePath))
                {
                    ImageInfo imageInfo = GetServiceImage(extentInfo
                                , ImageTheme.Satellite
                                , ServiceLevel.Premium);
                    //If imageInfo.image is null
                    imageInfo = GetImageFromRequest(imageInfo);

                    using (FileStream fileStream = File.Create(strImageFilePath))
                    {
                        fileStream.Write(imageInfo.Image, 0, imageInfo.Image.Length);
                        fileStream.Close();
                    }
                    File.WriteAllText(Path.ChangeExtension(strImageFilePath, ".jgw"), imageInfo.WorldData);
                }
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex, ex.Message);
            }
            finally
            {
                if (extentInfo != null)
                {
                    extentInfo = null;
                }
            }
            return strImageFilePath;
        }
        private static ImageInfo GetImageFromRequest(ImageInfo imageInfo)
        {
            System.Drawing.Image image = null;
            int SvcTimeOut = 10000;
            if ((imageInfo.Image == null || imageInfo.Image.Length == 0))
            {
                WebRequest webRequest = WebRequest.Create(imageInfo.ImageUrl);
                webRequest.Timeout = SvcTimeOut;
                WebResponse webResponse = webRequest.GetResponse();

                using (Stream inputStream = webResponse.GetResponseStream())
                {
                    image = System.Drawing.Image.FromStream(inputStream);
                    using (MemoryStream ms = new MemoryStream())
                    {
                        image.Save(ms, System.Drawing.Imaging.ImageFormat.Jpeg);
                        imageInfo.Image = ms.ToArray();
                    }
                }
            }
            else
            {
                using (MemoryStream memoryStream = new MemoryStream(imageInfo.Image, 0, imageInfo.Image.Length))
                {
                    image = System.Drawing.Image.FromStream(memoryStream);
                }
            }
            return imageInfo;
        }

        private ImageInfo GetServiceImage(ExtentInfo ExtentInfo
                                  , ImageTheme imageTheme
                                  , ServiceLevel serviceLevel)
        {
            ImageInfo imageInfo = null;
            try
            {
                string UserToken = "test";
                using (GeoLookup lookupServiceClient = CreateLookupServiceClient(UserToken))
                {
                    imageInfo = lookupServiceClient.GetImage(ExtentInfo, imageTheme, serviceLevel);
                }
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex, ex.Message);
            }
            return imageInfo;
        }
        internal GeoLookup CreateLookupServiceClient(string UserToken)
        {
            int SvcTimeOut = 0;            
            GeoLookup lookupServiceClient = new GeoLookup();            
            lookupServiceClient.Timeout = 10000;
            lookupServiceClient.UserIdentityValue = GetImageryIdentity(UserToken);

            return lookupServiceClient;
        }
        internal UserIdentity GetImageryIdentity(string UserToken)
        {
            UserIdentity userIdentity = new UserIdentity();
            userIdentity.UserName = UserToken;
            userIdentity.Password = "test";
            return userIdentity;
        }
        private void SyncClientZoomLevels()
        {
            ZoomLevelSet zoomLevels = new ZoomLevelSet();
            List<ZoomLevel> allowedZooms = new List<ZoomLevel>();
            allowedZooms.Add(zoomLevels.ZoomLevel01);
            allowedZooms.Add(zoomLevels.ZoomLevel02);
            allowedZooms.Add(zoomLevels.ZoomLevel03);
            allowedZooms.Add(zoomLevels.ZoomLevel04);
            allowedZooms.Add(zoomLevels.ZoomLevel05);
            allowedZooms.Add(zoomLevels.ZoomLevel06);
            allowedZooms.Add(zoomLevels.ZoomLevel07);
            allowedZooms.Add(zoomLevels.ZoomLevel08);
            allowedZooms.Add(zoomLevels.ZoomLevel09);
            allowedZooms.Add(zoomLevels.ZoomLevel10);
            allowedZooms.Add(zoomLevels.ZoomLevel11);
            allowedZooms.Add(zoomLevels.ZoomLevel12);
            allowedZooms.Add(zoomLevels.ZoomLevel13);
            allowedZooms.Add(zoomLevels.ZoomLevel14);
            allowedZooms.Add(zoomLevels.ZoomLevel15);
            allowedZooms.Add(zoomLevels.ZoomLevel16);
            allowedZooms.Add(zoomLevels.ZoomLevel17);
            allowedZooms.Add(zoomLevels.ZoomLevel18);
            allowedZooms.Add(zoomLevels.ZoomLevel19);
            allowedZooms.Add(zoomLevels.ZoomLevel20);
            allowedZooms.Add(new ZoomLevel(563));
            allowedZooms.Add(new ZoomLevel(281.5));

            Map1.SyncClientZoomLevels(allowedZooms);
        }
        private VirtualEarthOverlay GetVEOverLayer(string OverlayType, string UserID)
        {
            VirtualEarthOverlay veOverlay = new VirtualEarthOverlay(OverlayType.ToString());
            veOverlay.JavaScriptLibraryUri = new Uri("http://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.2");
            veOverlay.Name = OverlayType.ToString();
            switch (OverlayType)
            {
                case "Satellite":
                    veOverlay.VirtualEarthMapType = ThinkGeo.MapSuite.WebEdition.VirtualEarthMapType.Aerial;
                    break;     
                case "Hybrid":
                    veOverlay.VirtualEarthMapType = ThinkGeo.MapSuite.WebEdition.VirtualEarthMapType.Hybrid;
                    break;
                default:
                    veOverlay.VirtualEarthMapType = ThinkGeo.MapSuite.WebEdition.VirtualEarthMapType.Hybrid;
                    break;
            }

            veOverlay.IsBaseOverlay = true;
            veOverlay.IsVisible = true;
            veOverlay.IsVisibleInOverlaySwitcher = true;
            return veOverlay;
        }
        internal string ImageDir()
        {
            return @"c:\Temp\";
        }
        internal VirtualEarthLayer GetVELayer(ImageryOverlayType enumOverlayType, string astrUserID)
        {
            VirtualEarthLayer objVE = null;
            string strCachedPath = string.Empty;
            try
            {
                objVE = new VirtualEarthLayer();

                objVE.ServiceUrl = @"http://dycw01.dycominc.com/geoservice25/geolookup.asmx";
                objVE.ServiceTimeout = 60000;
                objVE.UserToken = astrUserID;
                switch (enumOverlayType)
                {
                    case ImageryOverlayType.Satellite:
                        objVE.MapType = ThinkGeo.MapSuite.Core.VirtualEarthMapType.Aerial;
                        strCachedPath = string.Format(@"{0}Bing\{1}", ImageDir(), ImageryOverlayType.Satellite.ToString());
                        break;
                    case ImageryOverlayType.Road:
                        objVE.MapType = ThinkGeo.MapSuite.Core.VirtualEarthMapType.Road;
                        strCachedPath = string.Format(@"{0}Bing\{1}", ImageDir(), ImageryOverlayType.Road.ToString());
                        break;
                    case ImageryOverlayType.Hybrid:
                        objVE.MapType = ThinkGeo.MapSuite.Core.VirtualEarthMapType.AerialWithLabels;
                        strCachedPath = string.Format(@"{0}Bing\{1}", ImageDir(), ImageryOverlayType.Hybrid.ToString());
                        break;
                    default:
                        objVE.MapType = ThinkGeo.MapSuite.Core.VirtualEarthMapType.AerialWithLabels;
                        strCachedPath = string.Format(@"{0}Bing\{1}", ImageDir(), ImageryOverlayType.Hybrid.ToString());
                        break;
                }

                if (!System.IO.Directory.Exists(strCachedPath))
                {
                    System.IO.Directory.CreateDirectory(strCachedPath);
                }
                objVE.CacheDirectory = strCachedPath;
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex, ex.Message);
            }
            return objVE;
        }

        private void CreateSketchOverlay()
        {
            LayerOverlay OverLaySketch = new LayerOverlay("Sketch", false, TileType.SingleTile);

            //Shape layer
            InMemoryFeatureLayer objShapeLayer = new InMemoryFeatureLayer();
            objShapeLayer.Open();
            objShapeLayer.FeatureSource.Projection = new Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString());
            objShapeLayer.Columns.Add(new FeatureSourceColumn("INMEMORY_SHAPE_TYPE"));
            objShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(155, 255, 255, 255), GeoColor.StandardColors.White, 1); ;
            objShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.FromArgb(180, 255, 255, 255), 10, GeoColor.StandardColors.White, 1);
            objShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.White, 7, true);
            objShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            objShapeLayer.Close();
            //Text layer
            InMemoryFeatureLayer objTextLayer = new InMemoryFeatureLayer();
            objTextLayer.Open();
            objTextLayer.FeatureSource.Projection = new Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString());
            objTextLayer.Columns.Add(new FeatureSourceColumn("INMEMORY_DESC"));
            objTextLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("INMEMORY_DESC"
                                            , new GeoFont("ARIEL", 11, DrawingFontStyles.Bold)
                                            , new GeoSolidBrush(GeoColor.FromArgb(255, GeoColor.SimpleColors.Red)));
            objTextLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            objTextLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.FittingPolygon = true;
            objTextLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.FittingPolygonFactor = 5000000;
            //objTextLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.YOffsetInPixel = 10;
            //objTextLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.XOffsetInPixel = 5;
            objTextLayer.DrawingQuality = DrawingQuality.HighQuality;
            //objTextLayer.DrawingMarginPercentage = 100;
            objTextLayer.Close();

            InMemoryFeatureLayer objLineTextLayer = new InMemoryFeatureLayer();
            objLineTextLayer.Open();
            objLineTextLayer.FeatureSource.Projection = new Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString());
            objLineTextLayer.Columns.Add(new FeatureSourceColumn("INMEMORY_DESC"));
            objLineTextLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("INMEMORY_DESC"
                                            , new GeoFont("ARIEL", 11, DrawingFontStyles.Bold)
                                            , new GeoSolidBrush(GeoColor.FromArgb(255, GeoColor.SimpleColors.Red)));
            objLineTextLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            
            OverLaySketch.Layers.Add("MAP_LAYER_SHAPE", objShapeLayer);
            OverLaySketch.Layers.Add("MAP_LAYER_TEXT", objTextLayer);
            OverLaySketch.Layers.Add("MAP_LAYER_LINE_TEXT", objTextLayer);
            OverLaySketch.IsVisibleInOverlaySwitcher = false;
            OverLaySketch.IsBaseOverlay = false;

            Map1.CustomOverlays.Add(OverLaySketch);

            SimpleMarkerOverlay objSimpleMarkerOverlay = new SimpleMarkerOverlay("MarkerSymbol");
            objSimpleMarkerOverlay.IsVisibleInOverlaySwitcher = false;
            objSimpleMarkerOverlay.IsBaseOverlay = false;
            Map1.CustomOverlays.Add(objSimpleMarkerOverlay);
        }
        private void AddShapeOverlay()
        {
            LayerOverlay objCustomOverLay = null;
            InMemoryFeatureLayer shapeLayer = null;
            InMemoryFeatureLayer textLayer = null;
            InMemoryFeatureLayer lineTextLayer = null;

            try
            {
                objCustomOverLay = (LayerOverlay)Map1.CustomOverlays["Sketch"];
                if (objCustomOverLay != null)
                {
                    shapeLayer = (InMemoryFeatureLayer)objCustomOverLay.Layers["MAP_LAYER_SHAPE"];
                    textLayer = (InMemoryFeatureLayer)objCustomOverLay.Layers["MAP_LAYER_TEXT"];
                    lineTextLayer = (InMemoryFeatureLayer)objCustomOverLay.Layers["MAP_LAYER_LINE_TEXT"];
                }
                // Add shape from EditOverlay to InMemoryFeatureLayer layers 
                foreach (string strEditShapeKey in Map1.EditOverlay.Features.GetKeys())
                {
                    Feature objFeatureEdit = Map1.EditOverlay.Features[strEditShapeKey];

                    string strShapeText = string.Empty;
                    if (objFeatureEdit.ColumnValues.ContainsKey("InMemoryDesc"))
                    {
                        objFeatureEdit.ColumnValues.TryGetValue("InMemoryDesc", out strShapeText);
                        objFeatureEdit.ColumnValues.Remove("InMemoryDesc");
                    }

                    Feature oFeature = new Feature(objFeatureEdit.GetWellKnownText());
                    shapeLayer.Open();
                    shapeLayer.EditTools.BeginTransaction();
                    shapeLayer.EditTools.Add(oFeature);
                    shapeLayer.EditTools.CommitTransaction();
                    shapeLayer.Close();
                    if (strShapeText.Length > 0)
                    {
                        
                        PointShape ptLabelCenter = new PointShape(oFeature.GetBoundingBox().GetCenterPoint().X
                                                    , oFeature.GetBoundingBox().GetCenterPoint().Y);

                        RectangleShape rect = ptLabelCenter.Buffer(10, GeographyUnit.Meter, DistanceUnit.Feet).GetBoundingBox();
                        Feature oTextFeature = new Feature(rect);
                        oTextFeature.ColumnValues.Add("INMEMORY_DESC", strShapeText);
                        textLayer.Open();
                        textLayer.EditTools.BeginTransaction();
                        textLayer.EditTools.Add(oTextFeature);
                        textLayer.EditTools.CommitTransaction();
                        textLayer.Close();
                    }
                }
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex, ex.Message);
            }
            finally
            {
                objCustomOverLay.Redraw();
            }
        }
        protected void Map1_BaseOverlayChanged(object sender, BaseOverlayChangedEventArgs e)
        {
            try
            {
                double CurrentMapScale = Map1.CurrentScale;
                Map1.EditOverlay.Features.Clear();                
                if (e.CurrentBaseOverlayId == "Aerial")
                {
                    //this.hdnEventhandler.Value = ((short)EventHandler.SvrAerialExtentChanged).ToString();
                    Map1.MapUnit = GeographyUnit.DecimalDegree;
                    RectangleShape convertMapExtent = ConvertProjection(Map1.CurrentExtent, ProjectionType.ToDecimalDegree);
                    CreateAerialOverlay(convertMapExtent, true);
                    Map1.ZoomTo(convertMapExtent.GetCenterPoint(), 563);
                }
                else
                {
                    CurrentMapScale = 1126;
                    RectangleShape convertMapExtent = (Map1.MapUnit == GeographyUnit.DecimalDegree) ? ConvertProjection(Map1.CurrentExtent, ProjectionType.ToMetric) : Map1.CurrentExtent;
                    Map1.MapUnit = GeographyUnit.Meter;
                    
                    Map1.ZoomTo(convertMapExtent.GetCenterPoint(), CurrentMapScale);
                }
              
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex, ex.Message);
            }
        }
        internal RectangleShape ConvertProjection(RectangleShape ShapeIn, ProjectionType ProjectionType)
        {

            return (RectangleShape)ConvertShapeProjection2(ShapeIn, ProjectionType);
        }
        private object ConvertShapeProjection2(object ShapeIn, ProjectionType ProjectionType)
        {
            object ShapeOut = null;
            try
            {
                using (Proj4Projection objProj4 = new Proj4Projection())
                {
                    objProj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
                    objProj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
                    objProj4.Open();
                    switch (ShapeIn.GetType().Name)
                    {
                        case "PointShape":
                        case "BaseShape":
                            ShapeOut = (ProjectionType == ProjectionType.ToMetric) ? objProj4.ConvertToExternalProjection((BaseShape)ShapeIn) : objProj4.ConvertToInternalProjection((BaseShape)ShapeIn);
                            break;
                        case "Feature":
                            ShapeOut = (ProjectionType == ProjectionType.ToMetric) ? objProj4.ConvertToExternalProjection((Feature)ShapeIn) : objProj4.ConvertToInternalProjection((Feature)ShapeIn);
                            break;
                        case "RectangleShape":
                            ShapeOut = (ProjectionType == ProjectionType.ToMetric) ? objProj4.ConvertToExternalProjection((RectangleShape)ShapeIn) : objProj4.ConvertToInternalProjection((RectangleShape)ShapeIn);
                            break;
                    }
                    objProj4.Close();
                }
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex, ex.Message);
            }
            return ShapeOut;
        }
        protected void btnTestGetBitmap_Click1(object sender, EventArgs e)
        {
            //AddShapeOverlay();
            //Bitmap bmWhiteLine = Map1.GetBitmap();
            //string strDir = @"C:\Submissions\TEMP\";
            //if (Directory.Exists(strDir))
            //{
            //    Directory.CreateDirectory(strDir);              
            //}
            //bmWhiteLine.Save(strDir + DateTime.Now.ToString("mmddyyyyhhmmss") + ".jpg");
            //bmWhiteLine.Dispose();
        }

        protected void Map1_OnDisposed(object sender, EventArgs e)
        {
            string strTest = string.Empty;
        }
        protected void btnSaveMap_Click(object sender, EventArgs e)
        {
            string strTest = string.Empty;
        }      
       
    
    }
}
